/**
  */ 

/* Includes ------------------------------------------------------------------*/
#include "stm32f4xx.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_spi.h"
#include "stm32f4xx_rcc.h"

#include "spi2.h"

/** @addtogroup SPI2
  * @{
  */  

void SPI2_LowLevel_DeInit(void);
void SPI2_LowLevel_Init(void);

/* Private functions ---------------------------------------------------------*/

/**
  * @brief  DeInitializes the peripherals used by the SPI FLASH driver.
  * @param  None
  * @retval None
  */
void SPI2_DeInit(void)
{
  SPI2_LowLevel_DeInit();
}

/**
  * @brief  Initializes the peripherals used by the SPI2 driver.
  * @param  None
  * @retval None
  */
void SPI2_Init(uint16_t prescaler)
{
  SPI_InitTypeDef  SPI_InitStructure;

  SPI2_LowLevel_Init();
    
  /*!< SPI configuration */
  SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
  SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
  SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
  SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;
  SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge;
  SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;
  SPI_InitStructure.SPI_BaudRatePrescaler = prescaler;
  // Speeds for APB1 running at 42MHz
  //  SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_16; // MHz
  //  SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_8; // MHz
  //  SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_4; // 10.5MHz
  //  SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_2; // 21MHz

  SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
  SPI_InitStructure.SPI_CRCPolynomial = 7;
  SPI_Init(SPI2, &SPI_InitStructure);

  /*!< Enable the sSPI2  */
  SPI_Cmd(SPI2, ENABLE);
}

/**
  * @brief  Sends data through the SPI interface and return nothing
  *         from the SPI bus.
  * @param  data: data to send.
  */
void SPI2_Send(uint16_t data)
{
	/*!< Loop while DR register in not empty */
	while ((SPI2->SR & SPI_I2S_FLAG_TXE) == RESET);

	SPI2->DR = data;

	// after last data send, use SPI2_WaitSend() to be sure transfer has finished
}

/**
  * @brief  Wait until the last transfer has finished
  * @retval The value of the received byte (throw away;) ).
  */
uint16_t SPI2_WaitSend(void)
{

	/*!< Loop while DR register in not empty */
	while ((SPI2->SR & SPI_I2S_FLAG_TXE) == RESET);

	/*!< Wait for transfer finished */
	while ((SPI2->SR & SPI_I2S_FLAG_BSY) == SET);
	return SPI2->DR; // this read resets RXNE flag
}

/**
  * @brief  Sends data through the SPI interface and return the data received
  *         from the SPI bus.
  * @param  data: data to send.
  * @retval The value of the received data.
  */
uint16_t SPI2_SendReceive(uint16_t data)
{
	/*!< Loop while DR register in not empty */
	// hk we do not need this if we can be sure last transmission ends before
	while ((SPI2->SR & SPI_I2S_FLAG_TXE) == RESET);

	/*!< Send byte through the SPI2 peripheral */
	SPI2->DR = data;

	/*!< Wait to receive data */
	while ((SPI2->SR & SPI_I2S_FLAG_RXNE) == RESET);

	/*!< Return data read from the SPI bus */
	// hk there is a 500ns pause between transfers.

	return SPI2->DR;
}

/**
  * @brief  Initializes the peripherals used by the SPI2 driver.
  * @param  None
  * @retval None
  */
void SPI2_LowLevel_Init(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;

  /*!< Enable the SPI clock */
  SPI2_CLK_INIT(SPI2_CLK, ENABLE);

  /*!< Enable GPIO clocks */
  RCC_AHB1PeriphClockCmd(SPI2_SCK_GPIO_CLK | SPI2_MISO_GPIO_CLK | SPI2_MOSI_GPIO_CLK , ENABLE);
  
  /*!< SPI pins configuration *************************************************/

  /*!< Connect SPI pins to AF5 */  
  GPIO_PinAFConfig(SPI2_SCK_GPIO_PORT, SPI2_SCK_SOURCE, SPI2_SCK_AF);
  GPIO_PinAFConfig(SPI2_MISO_GPIO_PORT, SPI2_MISO_SOURCE, SPI2_MISO_AF);
  GPIO_PinAFConfig(SPI2_MOSI_GPIO_PORT, SPI2_MOSI_SOURCE, SPI2_MOSI_AF);

  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_DOWN;
        
  /*!< SPI SCK pin configuration */
  GPIO_InitStructure.GPIO_Pin = SPI2_SCK_PIN;
  GPIO_Init(SPI2_SCK_GPIO_PORT, &GPIO_InitStructure);

  /*!< SPI MOSI pin configuration */
  GPIO_InitStructure.GPIO_Pin =  SPI2_MOSI_PIN;
  GPIO_Init(SPI2_MOSI_GPIO_PORT, &GPIO_InitStructure);

  /*!< SPI MISO pin configuration */
  GPIO_InitStructure.GPIO_Pin =  SPI2_MISO_PIN;
  GPIO_Init(SPI2_MISO_GPIO_PORT, &GPIO_InitStructure);
}

/**
  * @brief  DeInitializes the peripherals used by the SPI2 driver.
  * @param  None
  * @retval None
  */
void SPI2_LowLevel_DeInit(void)
{
//  GPIO_InitTypeDef GPIO_InitStructure;

  /*!< Disable the SPI2  ************************************************/
  SPI_Cmd(SPI2, DISABLE);
  
  /*!< DeInitializes the SPI2 *******************************************/
  SPI_I2S_DeInit(SPI2);
  
  /*!< SPI2 Periph clock disable ****************************************/
//  SPI2_CLK_INIT(SPI2_CLK, DISABLE);
      
  /*!< Configure all pins used by the SPI as input floating *******************/
//  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
//  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
//
//  GPIO_InitStructure.GPIO_Pin = SPI2_SCK_PIN;
//  GPIO_Init(SPI2_SCK_GPIO_PORT, &GPIO_InitStructure);
//
//  GPIO_InitStructure.GPIO_Pin = SPI2_MISO_PIN;
//  GPIO_Init(SPI2_MISO_GPIO_PORT, &GPIO_InitStructure);
//
//  GPIO_InitStructure.GPIO_Pin = SPI2_MOSI_PIN;
//  GPIO_Init(SPI2_MOSI_GPIO_PORT, &GPIO_InitStructure);
}

/**
  * @brief  Change speed of SPI interface
  * @param  New prescaler
  * @retval Old prescaler settings
  */
uint16_t SPI2_ChangeSpeed(uint16_t prescaler)
{
	uint16_t tmp_cr1, last_prescaler;

	// Disable SPI2
    SPI2->CR1 &= (uint16_t)~((uint16_t)SPI_CR1_SPE);

	tmp_cr1 = SPI2->CR1;

	// mask old prescaler value
	last_prescaler = tmp_cr1 & (uint16_t)0x0038;

	// zero prescaler bits
	tmp_cr1 &= (uint16_t)0xFFC7;

	// set new prescaler bits
	tmp_cr1 |= prescaler;

	// Enable SPI2 with new prescaler
	SPI2->CR1 = tmp_cr1 | SPI_CR1_SPE;

 	return last_prescaler;
}

/**
  * @brief  Change SPI datasize from 8 bit to 16 bit or vice versa
  * @param  new datasize
  * @retval Old datasize settings
  */
uint16_t SPI2_ChangeDatasize(uint16_t datasize)
{
	uint16_t tmp_cr1, last_datasize;

	// Disable SPI2
    SPI2->CR1 &= (uint16_t)~((uint16_t)SPI_CR1_SPE);

    // store old settings
	tmp_cr1 = SPI2->CR1;

	// mask old datasize value
	last_datasize = tmp_cr1 & (uint16_t)0x0800;

	// zero datasize bits
	tmp_cr1 &= (uint16_t)0xF7FF;

	// set new datasize bits
	tmp_cr1 |= datasize;

	// Enable SPI2 with new datasize
	SPI2->CR1 = tmp_cr1 | SPI_CR1_SPE;

 	return last_datasize;
}

/**
  * @}
  */

/**
  * @}
  */

